@@ -1,9 +1,13 @@
 import requests
+from datetime import datetime
 
 class PartialAccessToken():
     def __init__(self, access_token, client) -> None:
-        self.client: Client = client
+        self.client = client
         self.token = access_token
+
+    def revoke(self):
+        return self.client.revoke_token(self.token, token_type="access_token")
     
     def fetch_identify(self):
         response = requests.get("https://discord.com/api/v10/users/@me", headers={
@@ -74,9 +78,30 @@ class PartialAccessToken():
         elif response.status_code == 429: raise exceptions.RateLimited(f"You are being Rate Limited. Retry after: {response.json()['retry_after']}", retry_after=response.json()['retry_after'])
         else:
             raise exceptions.HTTPException(f"Unexpected HTTP {response.status_code}")
+    
+    def update_metadata(self, platform_name=None, username=None, **metadata):
+        def metadataTypeHook(item):
+            if type(item) == bool:
+                return 1 if item else 0
+            if type(item) == datetime:
+                return item.isoformat()
+            else: return item
+        response = requests.put(f"https://discord.com/api/v10/users/@me/applications/{self.client.id}/role-connection", headers={
+            "authorization": f"Bearer {self.token}"}, json={
+                "platform_name": platform_name,
+                "platform_username": username,
+                "metadata": {key: metadataTypeHook(value) for key, value in metadata.items()}
+            })
+
+        if response.ok:
+            return response.json()
+        elif response.status_code == 401: raise exceptions.Forbidden(f"this AccessToken does not have the nessasary scope.")
+        elif response.status_code == 429: raise exceptions.RateLimited(f"You are being Rate Limited. Retry after: {response.json()['retry_after']}", retry_after=response.json()['retry_after'])
+        else:
+            raise exceptions.HTTPException(f"Unexpected HTTP {response.status_code}")
 
 class AccessToken(PartialAccessToken):
-    def __init__(self, data: dict, client) -> None:
+    def __init__(self, data, client) -> None:
         super().__init__(data["access_token"], client)
 
         self.expires = data.get("expires_in")
@@ -84,6 +109,9 @@ class AccessToken(PartialAccessToken):
         self.refresh_token = data.get("refresh_token")
         self.webhook = data.get("webhook")
         self.guild = data.get("guild")
+    
+    def revoke_refresh_token(self):
+        return self.client.revoke_token(self.refresh_token, token_type="refresh_token")
 
 class Client():
     def __init__(self, id, secret, redirect, bot_token=None):
@@ -91,6 +119,10 @@ class Client():
         self.redirect_url = redirect
         self.__secret = secret
         self.__bot_token = bot_token
+
+    def update_linked_roles_metadata(self, metadata):
+        requests.put(f"https://discord.com/api/v10/applications/{self.id}/role-connections/metadata", headers={
+            "authorization": f"Bot {self.__bot_token}"}, json=metadata)
     
     def from_access_token(self, access_token):
         return PartialAccessToken(access_token, self)
@@ -124,13 +156,24 @@ class Client():
         response = requests.post("https://discord.com/api/v10/oauth2/token", data={
             "grant_type": "client_credentials", "scope": " ".join(scope)},
             auth=(self.id, self.__secret))
-        
         if response.ok:
             return AccessToken(response.json(), self)
         elif response.status_code == 400: raise exceptions.HTTPException("the scope, client id or client secret is invalid/don't match.")
         elif response.status_code == 429: raise exceptions.RateLimited(f"You are being Rate Limited. Retry after: {response.json()['retry_after']}", retry_after=response.json()['retry_after'])
         else:
             raise exceptions.HTTPException(f"Unexpected HTTP {response.status_code}")
+    
+    def revoke_token(self, token, token_type=None):
+        response = requests.post("https://discord.com/api/oauth2/token/revoke",
+            data={"token": token, "token_type_hint": token_type},
+            auth=(self.id, self.__secret))
+        print(response.status_code, response.text)
+        if response.ok:
+            return
+        elif response.status_code == 401: raise exceptions.Forbidden(f"this AccessToken does not have the nessasary scope.")
+        elif response.status_code == 429: raise exceptions.RateLimited(f"You are being Rate Limited. Retry after: {response.json()['retry_after']}", retry_after=response.json()['retry_after'])
+        else:
+            raise exceptions.HTTPException(f"Unexpected HTTP {response.status_code}")
 
 class exceptions():
     class BaseException(Exception):
